local ADDON, Addon = ...
local Journal = Addon.Journal
local BlipParent, MapFrame = WorldMapButton, WorldMapDetailFrame
local MISSING = ADDON_MISSING

local Map = Addon:NewModule('WorldMap', CreateFrame('Button', ADDON..'MapButton', BlipParent, 'PetTrackerMapButton'))
local Tooltip = Addon.MapTip(WorldMapFrame)
local L = Addon.Locals


--[[ Events ]]--

function Map:Startup()
	self:RegisterEvent('ZONE_CHANGED_NEW_AREA')
	self:SetScript('OnEvent', SetMapToCurrentZone)
	self:SetScript('OnUpdate', self.UpdateTip)
	self:SetScript('OnShow', self.UpdateBlips)
	self:SetScript('OnHide', self.HideTip)

	self.Flyout:SetScript('OnClick', self.ShowDrop)
	self:SetScript('OnClick', self.ToggleActive)
	self:SetPoint('TOPRIGHT', -6, -10)
	self:SetScale(1.1)
	self.blips = {}

	hooksecurefunc('WorldMap_ToggleSizeDown', function()
		self:SetScale(1.5)
	end)

	hooksecurefunc('WorldMap_ToggleSizeUp', function()
		self:SetScale(1.1)
	end)
end

function Map:TrackingChanged()
	if self:IsVisible() then 
		self:UpdateBlips()
	end
end


--[[ Blips ]]--

function Map:UpdateBlips()
	self:ResetBlips()

	if self:IsActive() then
		self:ShowBlips()
	end
end

function Map:ShowBlips()
	local floor = GetCurrentMapDungeonLevel() - (DungeonUsesTerrainMap() and 1 or 0)
	local species = Journal:GetCurrentSpecies()
	local width, height = MapFrame:GetSize()
	
	for specie, floors in pairs(species) do
		local spots = floors[floor]
		local best = Journal:GetBestOwned(specie)
			
		if spots and best <= self:GetFilter() then
			local icon = Journal:GetTypeIcon(specie)

			for x, y in gmatch(spots, '(%w+):(%w+)') do
				local x, y = self:ParseCoords(x, y)
				local blip = Addon.SpecieBlip(BlipParent)
				blip:SetPoint('CENTER', MapFrame, 'TOPLEFT', x * width, -y * height)
				blip.icon:SetTexture(icon)
				blip:SetSpecie(specie)
				blip.x, blip.y = x, y
				blip:Show()

				tinsert(self.blips, blip)
			end
		end
	end
end

function Map:ParseCoords(x, y)
	return tonumber(x, 16) / 1000, tonumber(y, 16) / 1000
end

function Map:ResetBlips()
	for _, blip in ipairs(self.blips) do
		blip:Release()
	end

	wipe(self.blips)
end


--[[ Tooltip ]]--

function Map:UpdateTip()
	Tooltip:Anchor(BlipParent, 'ANCHOR_CURSOR')

	for i, blip in ipairs(self.blips) do
		if blip:IsMouseOver() then
			local title, text = blip:GetTooltip()
			
			Tooltip:AddHeader(title)
			Tooltip:AddLine(text, 1,1,1, true)
		end
	end

	Tooltip:Display()
end

function Map:HideTip()
	Tooltip:Hide()
end


--[[ Active ]]--

function Map:ToggleActive()
	self:SetActive(not self:IsActive())
	self:UpdateBlips()
end

function Map:SetActive(active)
	local disabled = not active
	PetTracker_HideSpots = disabled
	
	self.Icon:SetDesaturated(disabled)
	self.Flyout:GetNormalTexture():SetDesaturated(disabled)
	self.Flyout:GetHighlightTexture():SetDesaturated(disabled)
end

function Map:IsActive()
	return not PetTracker_HideSpots
end


--[[ Filter ]]--

function Map:ShowDrop()
	local info = {
		{
			text = 'Pets',
			notCheckable = 1,
			isTitle = 1
    	}
	}
	
	local function addLine(name, quality)
		tinsert(info, {
			text = name,
			checked = quality == PetTracker_QualityFilter,
			func = function()
				PetTracker_QualityFilter = quality
				Map:UpdateBlips()
			end
    	})
	end
	
	addLine(ALL, Addon.MaxQuality)
	addLine(L.NotMaximized, Addon.MaxQuality - 1)
	addLine(MISSING, 0)

	EasyMenu(info, Addon.Blip.Drop, self, 0, 0)
end

function Map:GetFilter()
	return PetTracker_QualityFilter or Addon.MaxQuality
end
